@@ -67,10 +67,18 @@
   :group 'jabber)
 
 (defcustom jabber-post-connect-hooks '(jabber-send-current-presence
-				       jabber-muc-autojoin)
+				       jabber-muc-autojoin
+				       jabber-whitespace-ping-start
+				       jabber-vcard-avatars-find-current)
   "*Hooks run after successful connection and authentication.
 The functions should accept one argument, the connection object."
   :type 'hook
+  :options '(jabber-send-current-presence
+	     jabber-muc-autojoin
+	     jabber-whitespace-ping-start
+	     jabber-keepalive-start
+	     jabber-vcard-avatars-find-current
+	     jabber-autoaway-start)
   :group 'jabber-core)
 
 (defcustom jabber-pre-disconnect-hook nil
@@ -79,8 +87,9 @@ This might be due to failed authentication.  Check `*jabber-authenticated*'."
   :type 'hook
   :group 'jabber-core)
 
-(defcustom jabber-lost-connection-hook nil
-  "*Hooks run after involuntary disconnection"
+(defcustom jabber-lost-connection-hooks nil
+  "*Hooks run after involuntary disconnection.
+The functions are called with one argument: the connection object."
   :type 'hook
   :group 'jabber-core)
 
@@ -102,7 +111,7 @@ indefinitely.  See `password-cache' and `password-cache-expiry'."
   :type 'integer
   :group 'jabber-core)
 
-(defcustom jabber-roster-buffer "*-jabber-*"
+(defcustom jabber-roster-buffer "*-jabber-roster-*"
   "The name of the roster buffer"
   :type 'string
   :group 'jabber-core)
@@ -124,6 +133,12 @@ problems."
   "Return non-nil if SASL functions are available."
   (featurep 'sasl))
 
+(defvar jabber-account-history ()
+  "Keeps track of previously used jabber accounts")
+
+(defvar jabber-connection-type-history ()
+  "Keeps track of previously used connection types")
+
 ;; jabber-connect and jabber-connect-all should load jabber.el, not
 ;; just jabber-core.el, when autoloaded.
 
@@ -131,14 +146,25 @@ problems."
 (defun jabber-connect-all (&optional arg)
   "Connect to all configured Jabber accounts.
 See `jabber-account-list'.
-If no accounts are configured (or ARG supplied), call `jabber-connect' interactively."
+If no accounts are configured (or with prefix argument), call `jabber-connect' interactively.
+With many prefix arguments, one less is passed to `jabber-connect'."
   (interactive "P")
   (let ((accounts
 	 (remove-if (lambda (account)
 		      (cdr (assq :disabled (cdr account))))
 		    jabber-account-list)))
     (if (or (null accounts) arg)
-	(progn (setq current-prefix-arg nil) (call-interactively 'jabber-connect))
+	(let ((current-prefix-arg
+	       (cond
+		;; A number of C-u's; remove one, so to speak.
+		((consp arg)
+		 (if (> (car arg) 4)
+		     (list (/ (car arg) 4))
+		   nil))
+		;; Otherwise, we just don't care.
+		(t
+		 arg))))
+	  (call-interactively 'jabber-connect))
       ;; Only connect those accounts that are not yet connected.
       (let ((already-connected (mapcar #'jabber-connection-bare-jid jabber-connections))
 	    (connected-one nil))
@@ -165,7 +191,7 @@ If no accounts are configured (or ARG supplied), call `jabber-connect' interacti
 With prefix argument, register a new account.
 With double prefix argument, specify more connection details."
   (interactive
-   (let* ((jid (completing-read "Enter your JID: " jabber-account-list))
+   (let* ((jid (completing-read "Enter your JID: " jabber-account-list nil nil nil 'jabber-account-history))
 	  (entry (assoc jid jabber-account-list))
 	  (alist (cdr entry))
 	  password network-server port connection-type registerp)
@@ -186,6 +212,8 @@ With double prefix argument, specify more connection details."
 	 (setq network-server
 	       (read-string (format "Network server: (default `%s') " network-server)
 			    nil nil network-server))
+	 (when (zerop (length network-server))
+	   (setq network-server nil))
 	 (setq port
 	       (car
 		(read-from-string
@@ -200,7 +228,7 @@ With double prefix argument, specify more connection details."
 		    (mapcar (lambda (type)
 			      (cons (symbol-name (car type)) nil))
 			    jabber-connect-methods)
-		    nil t nil nil default)))))
+		    nil t nil 'jabber-connection-type-history default)))))
 	 (setq registerp (yes-or-no-p "Register new account? ")))
        (when (equal current-prefix-arg '(4))
 	 (setq registerp t))
@@ -209,6 +237,9 @@ With double prefix argument, specify more connection details."
 	     (jabber-jid-server jid)
 	     (jabber-jid-resource jid)
 	     registerp password network-server port connection-type))))
+
+  (require 'jabber)
+
   (if (member (list username
 		    server)
 	      (mapcar
@@ -267,7 +298,7 @@ With double prefix argument, specify more connection details."
 	(reason (plist-get state-data :disconnection-reason))
 	(ever-session-established (plist-get state-data :ever-session-established)))
     (unless expected
-      (run-hooks 'jabber-lost-connection-hook)
+      (run-hook-with-args 'jabber-lost-connection-hooks fsm)
       (message "%s@%s/%s: connection lost: `%s'"
 	       (plist-get state-data :username)
 	       (plist-get state-data :server)
@@ -281,6 +312,9 @@ With double prefix argument, specify more connection details."
       ;; of connections.
       (setq jabber-connections
 	    (delq fsm jabber-connections))
+      (when jabber-mode-line-mode
+        (jabber-mode-line-presence-update))
+      (jabber-display-roster)
       ;; And let the FSM sleep...
       (list state-data nil))))
 
@@ -313,16 +347,17 @@ With double prefix argument, specify more connection details."
     (:connected
      (let ((connection (cadr event))
 	   (registerp (plist-get state-data :registerp)))
-     
-       ;; TLS connections leave data in the process buffer, which
-       ;; the XML parser will choke on.
-       (with-current-buffer (process-buffer connection)
-	 (erase-buffer))
 
        (setq state-data (plist-put state-data :connection connection))
 
-       (set-process-filter connection (fsm-make-filter fsm))
-       (set-process-sentinel connection (fsm-make-sentinel fsm))
+       (when (processp connection)
+	 ;; TLS connections leave data in the process buffer, which
+	 ;; the XML parser will choke on.
+	 (with-current-buffer (process-buffer connection)
+	   (erase-buffer))
+
+	 (set-process-filter connection (fsm-make-filter fsm))
+	 (set-process-sentinel connection (fsm-make-sentinel fsm)))
 
        (list :connected state-data)))
 
@@ -738,21 +773,21 @@ With double prefix argument, specify more connection details."
   (interactive "P")
   (if arg
       (jabber-disconnect-one (jabber-read-account))
-      (unless *jabber-disconnecting*	; avoid reentry
-    (let ((*jabber-disconnecting* t))
-      (dolist (c jabber-connections)
-	(jabber-disconnect-one c t))
-      (setq jabber-connections nil)
+    (unless *jabber-disconnecting*	; avoid reentry
+      (let ((*jabber-disconnecting* t))
+	(run-hooks 'jabber-pre-disconnect-hook)
+	(dolist (c jabber-connections)
+	  (jabber-disconnect-one c t))
+	(setq jabber-connections nil)
 
-      (jabber-disconnected)
-      (when (interactive-p)
-	(message "Disconnected from Jabber server(s)"))))))
+	(jabber-disconnected)
+	(when (interactive-p)
+	  (message "Disconnected from Jabber server(s)"))))))
 
 (defun jabber-disconnect-one (jc &optional dont-redisplay)
   "Disconnect from one Jabber server.
 If DONT-REDISPLAY is non-nil, don't update roster buffer."
   (interactive (list (jabber-read-account)))
-  ;;(run-hooks 'jabber-pre-disconnect-hook)
   (fsm-send-sync jc :do-disconnect)
   (when (interactive-p)
     (message "Disconnected from %s"
@@ -990,8 +1025,8 @@ Return an fsm result list if it is."
 		   "")
 		 ">
 ")))
-    (jabber-send-string jc stream-header)
-    (jabber-log-xml jc "sending" stream-header)))
+    (jabber-log-xml jc "sending" stream-header)
+    (jabber-send-string jc stream-header)))
 
 (defun jabber-send-string (jc string)
   "Send STRING to the connection JC."
