@@ -1,6 +1,6 @@
 ;;; jabber-events.el --- Message events (JEP-0022) implementation
 
-;; Copyright (C) 2005  Magnus Henoch
+;; Copyright (C) 2005, 2008  Magnus Henoch
 
 ;; Author: Magnus Henoch <mange@freemail.hu>
 
@@ -19,6 +19,7 @@
 ;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
+(require 'jabber-autoloads)
 (require 'cl)
 
 (defgroup jabber-events nil 
@@ -124,9 +125,14 @@ and it hasn't been sent before."
     (when (and jabber-events-confirm-displayed
 	       (not jabber-events-display-confirmed)
 	       (memq 'displayed jabber-events-requested)
+	       ;; XXX: if jabber-events-requested is non-nil, how can
+	       ;; jabber-chatting-with be nil?  See
+	       ;; http://sourceforge.net/tracker/index.php?func=detail&aid=1872560&group_id=88346&atid=586350
+	       jabber-chatting-with
 	       ;; don't send to bare jids
 	       (jabber-jid-resource jabber-chatting-with))
       (jabber-send-sexp 
+       jabber-buffer-connection
        `(message 
 	 ((to . ,jabber-chatting-with))
 	 (x ((xmlns . "jabber:x:event"))
@@ -140,6 +146,7 @@ and it hasn't been sent before."
 	       jabber-chatting-with
 	       (not (eq composing-now jabber-events-composing-sent)))
       (jabber-send-sexp 
+       jabber-buffer-connection
        `(message 
 	 ((to . ,jabber-chatting-with))
 	 (x ((xmlns . "jabber:x:event"))
@@ -152,7 +159,7 @@ and it hasn't been sent before."
 ;; Add function last in chain, so a chat buffer is already created.
 (add-to-list 'jabber-message-chain 'jabber-handle-incoming-message-events t)
 
-(defun jabber-handle-incoming-message-events (xml-data)
+(defun jabber-handle-incoming-message-events (jc xml-data)
   (when (and (not (jabber-muc-message-p xml-data))
 	     (get-buffer (jabber-chat-get-buffer (jabber-xml-get-attribute xml-data 'from))))
     (with-current-buffer (jabber-chat-get-buffer (jabber-xml-get-attribute xml-data 'from))
@@ -191,6 +198,7 @@ and it hasn't been sent before."
 	  (flet ((send-notification 
 		  (type)
 		  (jabber-send-sexp 
+		   jc
 		   `(message 
 		     ((to . ,(jabber-xml-get-attribute xml-data 'from)))
 		     (x ((xmlns . "jabber:x:event"))
