@@ -1,7 +1,7 @@
 ;; jabber-keepalive.el - try to detect lost connection
 
+;; Copyright (C) 2004, 2008 - Magnus Henoch - mange@freemail.hu
 ;; Copyright (C) 2007 - Detlev Zundel - dzu@gnu.org
-;; Copyright (C) 2004 - Magnus Henoch - mange@freemail.hu
 
 ;; This file is a part of jabber.el.
 
@@ -19,19 +19,25 @@
 ;; along with this program; if not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-;;; These keepalive functions send a jabber:iq:time request to the
+
+;;;; Keepalive - send something to the server and see if it answers
+;;;
+;;; These keepalive functions send a urn:xmpp:ping request to the
 ;;; server every X minutes, and considers the connection broken if
 ;;; they get no answer within Y seconds.
 
+;;;###autoload
 (defgroup jabber-keepalive nil
   "Keepalive functions try to detect lost connection"
   :group 'jabber)
 
+;;;###autoload
 (defcustom jabber-keepalive-interval 600
   "Interval in seconds between connection checks."
   :type 'integer
   :group 'jabber-keepalive)
 
+;;;###autoload
 (defcustom jabber-keepalive-timeout 20
   "Seconds to wait for response from server."
   :type 'integer
@@ -49,8 +55,13 @@
 (defvar jabber-keepalive-debug nil
   "Log keepalive traffic when non-nil")
 
+;;;###autoload
 (defun jabber-keepalive-start (&optional jc)
   "Activate keepalive.
+That is, regularly send a ping request to the server, and
+disconnect if it doesn't answer.  See `jabber-keepalive-interval'
+and `jabber-keepalive-timeout'.
+
 The JC argument makes it possible to add this function to
 `jabber-post-connect-hooks'; it is ignored.  Keepalive is activated
 for all accounts regardless of the argument."
@@ -85,7 +96,8 @@ for all accounts regardless of the argument."
     ;; Whether we get an error or not is not interesting.
     ;; Getting a response at all is.
     (jabber-send-iq c nil "get"
-		    '(query ((xmlns . "jabber:iq:time")))
+		    ;; "ping" is XEP-0199
+		    '(query ((xmlns . "urn:xmpp:ping")))
 		    'jabber-keepalive-got-response nil
 		    'jabber-keepalive-got-response nil)))
 
@@ -108,9 +120,62 @@ for all accounts regardless of the argument."
 	     (current-time-string)
 	     (plist-get (fsm-get-state-data c) :server))
 
-    (run-hooks jabber-lost-connection-hook)
+    (run-hooks jabber-lost-connection-hooks c)
     (jabber-disconnect-one c nil)))
 
+;;;; Whitespace pings - less traffic, no error checking on our side
+;;;
+;;; Openfire needs something like this, but I couldn't bring myself to
+;;; enable keepalive by default... Whitespace pings are light and
+;;; unobtrusive.
+
+;;;###autoload
+(defcustom jabber-whitespace-ping-interval 30
+  "Send a space character to the server with this interval, in seconds.
+
+This is a traditional remedy for a number of problems: to keep NAT
+boxes from considering the connection dead, to have the OS discover
+earlier that the connection is lost, and to placate servers which rely
+on the client doing this, e.g. Openfire.
+
+If you want to verify that the server is able to answer, see
+`jabber-keepalive-start' for another mechanism."
+  :type '(integer :tag "Interval in seconds")
+  :group 'jabber-core)
+
+(defvar jabber-whitespace-ping-timer nil
+  "Timer object for whitespace pings")
+
+;;;###autoload
+(defun jabber-whitespace-ping-start (&optional jc)
+  "Start sending whitespace pings at regular intervals.
+See `jabber-whitespace-ping-interval'.
+
+The JC argument is ignored; whitespace pings are enabled for all
+accounts."
+  (interactive)
+
+  (when jabber-whitespace-ping-timer
+    (jabber-whitespace-ping-stop))
+
+  (setq jabber-whitespace-ping-timer
+	(run-with-timer 5
+			jabber-whitespace-ping-interval
+			'jabber-whitespace-ping-do))
+  (add-hook 'jabber-post-disconnect-hook 'jabber-whitespace-ping-stop))
+
+(defun jabber-whitespace-ping-stop ()
+  "Deactivate whitespace pings"
+  (interactive)
+
+  (when jabber-whitespace-ping-timer
+    (jabber-cancel-timer jabber-whitespace-ping-timer)
+    (setq jabber-whitespace-ping-timer nil)))
+
+(defun jabber-whitespace-ping-do ()
+  (dolist (c jabber-connections)
+    (jabber-send-string c " ")))
+
 (provide 'jabber-keepalive)
 
 ;;; arch-tag: d19ca743-75a1-475f-9217-83bd18012146
