@@ -106,7 +106,7 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
     ;; This is the function that does the actual updating and
     ;; redrawing of the roster.
     (jabber-roster-update jc new-items changed-items deleted-items)
-      
+
     (if (and id (string= type "set"))
 	(jabber-send-iq jc nil "result" nil
 			nil nil nil nil id)))
@@ -217,7 +217,7 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
 				    oldstatus
 				    newstatus
 				    (plist-get resource-plist 'status)
-				    (funcall jabber-alert-presence-message-function 
+				    (funcall jabber-alert-presence-message-function
 					     buddy
 					     oldstatus
 					     newstatus
@@ -292,24 +292,20 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
 ;;;###autoload
 (defun jabber-send-presence (show status priority)
   "Set presence for all accounts."
-  (interactive (list (completing-read "show:"
-				      '(("" . nil)
-					("away" . nil)
-					("xa" . nil)
-					("dnd" . nil)
-					("chat" . nil))
-				      nil t nil 'jabber-presence-history)
-		     (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*)
-		     (read-string "priority: " (progn
-						 (unless *jabber-current-priority*
-						   (setq *jabber-current-priority*
-							 jabber-default-priority))
-						 (int-to-string *jabber-current-priority*)))))
-  (if (numberp priority)
-      (setq priority (int-to-string priority)))
-  (setq *jabber-current-status* status)
-  (setq *jabber-current-show* show)
-  (setq *jabber-current-priority* (string-to-number priority))
+  (interactive
+   (list
+    (completing-read "show: " '("" "away" "xa" "dnd" "chat")
+		     nil t nil 'jabber-presence-history)
+    (jabber-read-with-input-method "status message: " *jabber-current-status*
+				   '*jabber-status-history*)
+    (read-string "priority: " (int-to-string (if *jabber-current-priority*
+						 *jabber-current-priority*
+					       jabber-default-priority)))))
+
+  (setq *jabber-current-show* show *jabber-current-status* status)
+  (setq *jabber-current-priority*
+	(if (numberp priority) priority (string-to-number priority)))
+
   (let (subelements-map)
     ;; For each connection, we use a different set of subelements.  We
     ;; cache them, to only generate them once.
@@ -319,14 +315,17 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
       (let ((subelements (jabber-presence-children jc)))
 	(aput 'subelements-map jc subelements)
 	(jabber-send-sexp-if-connected jc `(presence () ,@subelements))))
+
     ;; Then send presence to groupchats
-    (dolist (groupchat *jabber-active-groupchats*)
-      (let* ((buffer (get-buffer (jabber-muc-get-buffer (car groupchat))))
+    (dolist (gc *jabber-active-groupchats*)
+      (let* ((buffer (get-buffer (jabber-muc-get-buffer (car gc))))
 	     (jc (when buffer
 		   (buffer-local-value 'jabber-buffer-connection buffer)))
 	     (subelements (cdr (assq jc subelements-map))))
 	(when jc
-	  (jabber-send-sexp-if-connected jc `(presence ((to . ,(car groupchat))) ,@subelements))))))
+	  (jabber-send-sexp-if-connected jc `(presence ((to . ,(car gc)))
+						       ,@subelements))))))
+
   (jabber-display-roster))
 
 (defun jabber-presence-children (jc)
@@ -377,7 +376,7 @@ TYPE is one of:
 			    ("unsubscribed"))
 			  nil t nil 'jabber-presence-history "online")))
   (cond
-   ((member type '("probe" "unavailable" 
+   ((member type '("probe" "unavailable"
 		   "subscribe" "unsubscribe"
 		   "subscribed" "unsubscribed"))
     (jabber-send-sexp jc `(presence ((to . ,jid)
@@ -395,52 +394,55 @@ TYPE is one of:
 (defun jabber-send-away-presence (&optional status)
   "Set status to away.
 With prefix argument, ask for status message."
-  (interactive (list
-		(when current-prefix-arg
-		  (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*))))
-  (jabber-send-presence "away" (if status status *jabber-current-status*) *jabber-current-priority*))
+  (interactive
+   (list
+    (when current-prefix-arg
+      (jabber-read-with-input-method
+       "status message: " *jabber-current-status* '*jabber-status-history*))))
+  (jabber-send-presence "away" (if status status *jabber-current-status*)
+			*jabber-current-priority*))
 
+;; XXX code duplication!
 (defun jabber-send-xa-presence (&optional status)
   "Send extended away presence.
 With prefix argument, ask for status message."
-  (interactive (list
-        (when current-prefix-arg
-          (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*))))
-  (jabber-send-presence "xa" (if status status *jabber-current-status*) *jabber-current-priority*))
+  (interactive
+   (list
+    (when current-prefix-arg
+      (jabber-read-with-input-method
+       "status message: " *jabber-current-status* '*jabber-status-history*))))
+  (jabber-send-presence "xa" (if status status *jabber-current-status*)
+			*jabber-current-priority*))
 
 ;;;###autoload
-(defun jabber-send-default-presence (&optional jc)
+(defun jabber-send-default-presence (&optional ignore)
   "Send default presence.
-Default presence is specified by `jabber-default-priority', `jabber-default-show',
-and `jabber-default-status'."
+Default presence is specified by `jabber-default-show',
+`jabber-default-status', and `jabber-default-priority'."
   (interactive)
-  ;; jc is ignored.  It's only there so this function can be in
-  ;; jabber-post-connect-hooks.
   (jabber-send-presence
-   jabber-default-show
-   (if (not (string= jabber-default-status "")) jabber-default-status *jabber-current-status*)
-   jabber-default-priority))
+   jabber-default-show jabber-default-status jabber-default-priority))
 
-(defun jabber-send-current-presence (&optional jc)
+(defun jabber-send-current-presence (&optional ignore)
   "(Re-)send current presence.
 That is, if presence has already been sent, use current settings,
-else send defaults (see `jabber-send-default-presence')."
+otherwise send defaults (see `jabber-send-default-presence')."
   (interactive)
-  ;; jc is ignored.  It's only there so this function can be in
-  ;; jabber-post-connect-hooks.
   (if *jabber-current-show*
-      (jabber-send-presence *jabber-current-show* *jabber-current-status* *jabber-current-priority*)
+      (jabber-send-presence *jabber-current-show* *jabber-current-status*
+			    *jabber-current-priority*)
     (jabber-send-default-presence)))
 
-(add-to-list 'jabber-jid-roster-menu
-	     (cons "Send subscription request" 'jabber-send-subscription-request))
+(add-to-list 'jabber-jid-roster-menu (cons "Send subscription request"
+					   'jabber-send-subscription-request))
 (defun jabber-send-subscription-request (jc to &optional request)
-  "send a subscription request to jid, showing him your request text, if specified"
+  "send a subscription request to jid, showing him your request
+text, if specified"
   (interactive (list (jabber-read-account)
 		     (jabber-read-jid-completing "to: ")
 		     (jabber-read-with-input-method "request: ")))
   (jabber-send-sexp jc
-		    `(presence 
+		    `(presence
 		      ((to . ,to)
 		       (type . "subscribe"))
 		      ,@(when (and request (> (length request) 0))
@@ -469,7 +471,7 @@ else send defaults (see `jabber-send-default-presence')."
 		 (list account
 		       jid (jabber-read-with-input-method (format "Name: (default `%s') " name) nil nil name)
 		       (delete ""
-			       (completing-read-multiple 
+			       (completing-read-multiple
 				(format
 				 "Groups, comma-separated: (default %s) "
 				 (if groups
@@ -483,14 +485,14 @@ else send defaults (see `jabber-send-default-presence')."
   ;; If new fields are added to the roster XML structure in a future standard,
   ;; they will be clobbered by this function.
   ;; XXX: specify account
-  (jabber-send-iq jc nil "set" 
+  (jabber-send-iq jc nil "set"
 		  (list 'query (list (cons 'xmlns "jabber:iq:roster"))
 			(list 'item (append
 				     (list (cons 'jid (symbol-name jid)))
 				     (if (and name (> (length name) 0))
 					 (list (cons 'name name))))
 			      (mapcar #'(lambda (x) `(group () ,x))
-				      groups))) 
+				      groups)))
 		  #'jabber-report-success "Roster item change"
 		  #'jabber-report-success "Roster item change"))
 
@@ -518,6 +520,33 @@ Signal an error if there is no JID at point."
 	(jabber-roster-delete account jid-at-point)
       (error "No contact at point"))))
 
+(defun jabber-roster-delete-group-from-jids (jc jids group)
+  "Delete group `group' from all JIDs"
+  (interactive)
+  (dolist (jid jids)
+    (jabber-roster-change
+     jc jid (get jid 'name)
+     (remove-if-not (lambda (g) (not (string= g group)))
+		    (get jid 'groups)))))
+
+(defun jabber-roster-edit-group-from-jids (jc jids group)
+  "Edit group `group' from all JIDs"
+  (interactive)
+  (let ((new-group
+	 (jabber-read-with-input-method
+	  (format "New group: (default `%s') " group) nil nil group)))
+    (dolist (jid jids)
+      (jabber-roster-change
+       jc jid (get jid 'name)
+       (remove-duplicates
+	(mapcar
+	 (lambda (g) (if (string= g group)
+			 new-group
+		       g))
+	 (get jid 'groups))
+	:test 'string=)))))
+
+
 (provide 'jabber-presence)
 
 ;;; arch-tag: b8616d4c-dde8-423e-86c7-da7b4928afc3
