@@ -1,7 +1,7 @@
 ;; jabber-register.el - registration according to JEP-0077
 
+;; Copyright (C) 2003, 2004, 2007 - Magnus Henoch - mange@freemail.hu
 ;; Copyright (C) 2002, 2003, 2004 - tom berger - object@intelectronica.net
-;; Copyright (C) 2003, 2004 - Magnus Henoch - mange@freemail.hu
 
 ;; This file is a part of jabber.el.
 
@@ -24,16 +24,17 @@
 
 (add-to-list 'jabber-jid-service-menu
 	     (cons "Register with service" 'jabber-get-register))
-(defun jabber-get-register (to)
+(defun jabber-get-register (jc to)
   "Send IQ get request in namespace \"jabber:iq:register\"."
-  (interactive (list (jabber-read-jid-completing "Register with: ")))
-  (jabber-send-iq to
+  (interactive (list (jabber-read-account)
+		     (jabber-read-jid-completing "Register with: ")))
+  (jabber-send-iq jc to
 		  "get"
 		  '(query ((xmlns . "jabber:iq:register")))
 		  #'jabber-process-data #'jabber-process-register-or-search
 		  #'jabber-report-success "Registration"))
 
-(defun jabber-process-register-or-search (xml-data)
+(defun jabber-process-register-or-search (jc xml-data)
   "Display results from jabber:iq:{register,search} query as a form."
 
   (let ((query (jabber-iq-query xml-data))
@@ -44,39 +45,42 @@
 	       ((string= (jabber-iq-xmlns xml-data) "jabber:iq:search")
 		'search)
 	       (t
-		(error "Namespace %s not handled by jabber-process-register-or-search" (jabber-iq-xmlns xml-data))))))
+		(error "Namespace %s not handled by jabber-process-register-or-search" (jabber-iq-xmlns xml-data)))))
+	(register-account
+	 (plist-get (fsm-get-state-data jc) :registerp))
+	(username
+	 (plist-get (fsm-get-state-data jc) :username))
+	(server
+	 (plist-get (fsm-get-state-data jc) :server)))
 	       
     (cond
      ((eq type 'register)
       ;; If there is no `from' attribute, we are registering with the server
-      (jabber-init-widget-buffer (or (jabber-xml-get-attribute xml-data 'from) jabber-server)))
+      (jabber-init-widget-buffer (or (jabber-xml-get-attribute xml-data 'from) 
+				     server)))
 
      ((eq type 'search)
       ;; no such thing here
       (jabber-init-widget-buffer (jabber-xml-get-attribute xml-data 'from))))
 
-    (widget-insert (if (eq type 'register) "Register with " "Search ") jabber-submit-to "\n\n")
-    (when (and (eq type 'register)
-	     jabber-register-p)
-	(widget-insert "Don't change the username here unless you also change ")
-	(widget-create 'link
-		       :notify (lambda (&rest ignore)
-				 (customize-variable 'jabber-username))
-		       "jabber-username")
-	(widget-insert ".\n\n"))
+    (setq jabber-buffer-connection jc)
 
+    (widget-insert (if (eq type 'register) "Register with " "Search ") jabber-submit-to "\n\n")
+    
     (dolist (x (jabber-xml-get-children query 'x))
       (when (string= (jabber-xml-get-attribute x 'xmlns) "jabber:x:data")
 	(setq have-xdata t)
 	;; If the registration form obeys JEP-0068, we know
 	;; for sure how to put a default username in it.
 	(jabber-render-xdata-form x
-				  (if (and jabber-register-p
+				  (if (and register-account
 					   (string= (jabber-xdata-formtype x) "jabber:iq:register"))
-				      (list (cons "username" jabber-username))
+				      (list (cons "username" username))
 				    nil))))
     (if (not have-xdata)
-	(jabber-render-register-form query))
+	(jabber-render-register-form query
+				     (when register-account
+				       username)))
 
     (widget-create 'push-button :notify (if (eq type 'register)
 					    #'jabber-submit-register
@@ -91,11 +95,12 @@
 (defun jabber-submit-register (&rest ignore)
   "Submit registration input.  See `jabber-process-register-or-search'."
   
-  (let ((handler (if jabber-register-p 
-		     #'jabber-process-register-secondtime
-		   #'jabber-report-success))
-	(text (concat "Registration with " jabber-submit-to)))
-    (jabber-send-iq jabber-submit-to
+  (let* ((registerp (plist-get (fsm-get-state-data jabber-buffer-connection) :registerp))
+	 (handler (if registerp
+		      #'jabber-process-register-secondtime
+		    #'jabber-report-success))
+	 (text (concat "Registration with " jabber-submit-to)))
+    (jabber-send-iq jabber-buffer-connection jabber-submit-to
 		    "set"
 
 		    (cond
@@ -107,29 +112,27 @@
 			      ,(jabber-parse-xdata-form)))
 		     (t
 		      (error "Unknown form type: %s" jabber-form-type)))
-		    handler (if jabber-register-p 'success text)
-		    handler (if jabber-register-p 'failure text)))
+		    handler (if registerp 'success text)
+		    handler (if registerp 'failure text)))
 
   (message "Registration sent"))
 
-(defun jabber-process-register-secondtime (xml-data closure-data)
+(defun jabber-process-register-secondtime (jc xml-data closure-data)
   "Receive registration success or failure.
 CLOSURE-DATA is either 'success or 'error."
   (cond
    ((eq closure-data 'success)
-    (message "Registration successful.  Your JID is %s@%s.  You may now connect to the server."
-	     jabber-username jabber-server)
-    (sit-for 3)
-    (jabber-disconnect))
+    (message "Registration successful.  You may now connect to the server."))
    (t
-    (jabber-report-success xml-data "Account registration")
-    (sit-for 3))))
+    (jabber-report-success jc xml-data "Account registration")))
+  (sit-for 3)
+    (jabber-disconnect-one jc))
 
 (defun jabber-remove-register (&rest ignore)
   "Cancel registration.  See `jabber-process-register-or-search'."
 
   (if (yes-or-no-p (concat "Are you sure that you want to cancel your registration to " jabber-submit-to "? "))
-      (jabber-send-iq jabber-submit-to
+      (jabber-send-iq jabber-buffer-connection jabber-submit-to
 		      "set"
 		      '(query ((xmlns . "jabber:iq:register"))
 			      (remove))
