@@ -154,12 +154,13 @@ configure a Google Talk account like this:
 (require 'jabber-xmessage)
 (require 'jabber-wmii)
 (require 'jabber-osd)
+(require 'jabber-awesome)
 
 (require 'jabber-autoloads)
 
 ;;;###autoload
 (defvar *jabber-current-status* nil
-  "the users current presence staus")
+  "the users current presence status")
 
 ;;;###autoload
 (defvar *jabber-current-show* nil
@@ -217,16 +218,15 @@ Contents of process buffers might be useful for debugging."
    (nil . jabber-roster-user-offline))
  "Mapping from presence types to faces")
 
-;;;###autoload
 (defconst jabber-presence-strings
-  '(("" . "Online")
-    ("away" . "Away")
-    ("xa" . "Extended Away")
-    ("dnd" . "Do not Disturb")
-    ("chat" . "Chatty")
-    ("error" . "Error")
-    (nil . "Offline"))
-  "Mapping from presence types to readable strings")
+  `(("" . ,(jabber-propertize "Online" 'face 'jabber-roster-user-online))
+    ("away" . ,(jabber-propertize "Away" 'face 'jabber-roster-user-away))
+    ("xa" . ,(jabber-propertize "Extended Away" 'face 'jabber-roster-user-xa))
+    ("dnd" . ,(jabber-propertize "Do not Disturb" 'face 'jabber-roster-user-dnd))
+    ("chat" . ,(jabber-propertize "Chatty" 'face 'jabber-roster-user-chatty))
+    ("error" . ,(jabber-propertize "Error" 'face 'jabber-roster-user-error))
+    (nil . ,(jabber-propertize "Offline" 'face 'jabber-roster-user-offline)))
+  "Mapping from presence types to readable, colorized strings")
 
 ;;;###autoload
 (defun jabber-customize ()
